/*
 *
 *	(c) Petr Susil 2006 <susa@atrey.karlin.mff.cuni.cz>
 *
 *	This software may be freely distributed and used according to the terms
 *	of the GNU General Public License.
 */

#include "hash_algo.h"
#include <stdio.h>

#ifdef SDBM
/*sdbm
 * this algorithm was created for sdbm (a public-domain reimplementation of ndbm) database library.
 * it was found to do well in scrambling bits, causing better distribution of the keys and fewer splits. 
 * it also happens to be a good general hashing function with good distribution. 
 * the actual function is hash(i) = hash(i - 1) * 65599 + str[i]; 
 * what is included below is the faster version used in gawk. 
 * [there is even a faster, duff-device version] the magic constant 65599 was picked out of thin air
 * while experimenting with different constants, and turns out to be a prime. 
 * this is one of the algorithms used in berkeley db (see sleepycat) and elsewhere.
 */
 unsigned long stringhash( const unsigned char *str)
{
  unsigned long hash = 0;
  int c;
  
  while ((c = *str++))
    hash = c + (hash << 6) + (hash << 16) - hash;
  
  return hash;
}
#endif

#ifdef DJB2
/* djb2
 * this algorithm (k=33) was first reported by dan bernstein many years ago in comp.lang.c. 
 * another version of this algorithm (now favored by bernstein) uses xor: 
 * hash(i) = hash(i - 1) * 33 ^ str[i]; 
 * the magic of number 33 (why it works better than many other constants, prime or not) 
 * has never been adequately explained.
 */
 unsigned long stringhash(const unsigned char *str)
{
  unsigned long hash = 5381;
  int c;
  while ((c = *str++))
    hash = ((hash << 5) + hash) + c; /* hash * 33 + c */
  return hash;
}

#endif

#ifdef SHA1
#define SHA_OFFSET 0
#include <string.h>
#include <openssl/sha.h>
unsigned long stringhash(const unsigned char *str)
{
  unsigned char md[SHA_DIGEST_LENGTH];
  unsigned char* sha = 
    SHA1 ( 
	  str, 
	  strlen((char*)str), 
	  md);
  unsigned long hash = * ((unsigned long*) sha + SHA_OFFSET);
  
  return hash;
}
#undef SHA_OFFSET
#endif

#ifdef MD5
#define MD5_OFFSET 0
#include <string.h>
#include <openssl/md5.h>
unsigned long stringhash(const unsigned char *str)
{
  unsigned char md[MD5_DIGEST_LENGTH];
  unsigned char* sha = 
    MD5 ( 
	  str, 
	  strlen((char*)str), 
	  md);
  unsigned long hash = * ((unsigned long*) sha + MD5_OFFSET);
  return hash;
}
#undef MD5_OFFSET
#endif




#if 0
/*Thomas Wang's 32 bit Mix Function*/
 unsigned int inthash(unsigned int key)
{
  key += ~(key << 15);
  key ^=  (key >>> 10);
  key +=  (key << 3);
  key ^=  (key >>> 6);
  key += ~(key << 11);
  key ^=  (key >>> 16);
  return key;
}
#else
/* Robert Jenkins' 32 bit Mix Function */
 unsigned int inthash(unsigned int key)
{
  //  printf("heshing key=%X\n",key);
  key += (key << 12);
  key ^= (key >> 22);
  key += (key << 4);
  key ^= (key >> 9);
  key += (key << 10);
  key ^= (key >> 2);
  key += (key << 7);
  key ^= (key >> 12);
  //  printf("hesh=%X\n",key);
  return key;
}
#endif
