/*
 *
 *	(c) Petr Susil 2006 <susa@atrey.karlin.mff.cuni.cz>
 *
 *	This software may be freely distributed and used according to the terms
 *	of the GNU General Public License.
 */

#ifndef SPARSE_BUILD_VISIBLE_AT_H
#define SPARSE_BUILD_VISIBLE_AT_H

static void walk_statement(struct statement* stmt, int level);
static void traverse_statement(struct statement* stmt, int line, int pos);

static void traverse_expression_symbol(struct expression* expr, int line, int pos)
{
  if (expr->symbol)  {
    struct symbol* sym=expr->symbol->ctype.base_type;
    if (sym)
      {
	if (sym->type==SYM_FN)	  {
	  add_function_use(act_fn,expr); 
	}
	else	  {
	  add_variable_use(act_fn,expr);
	}
      }
  }
  return ;
}

static void traverse_expression(struct expression* expr, int line, int pos)
{
  if (!expr) return;
  switch (expr->type)
    {
    case EXPR_SYMBOL:    
      traverse_expression_symbol(expr,line,pos); return;
    case EXPR_BINOP:
    case EXPR_LOGICAL:
    case EXPR_COMMA:
    case EXPR_COMPARE:
    case EXPR_ASSIGNMENT:
	  traverse_expression(expr->left,line,pos);
	  traverse_expression(expr->right,line,pos);
	  return ;
    case EXPR_VALUE:
    case EXPR_FVALUE:    return ;
    case EXPR_STRING:    return ;
    case EXPR_PREOP:
    case EXPR_POSTOP:   traverse_expression(expr->unop,line,pos);  return ;
    case EXPR_CAST:          return traverse_expression(expr->cast_expression,line,pos);
    case EXPR_IMPLIED_CAST:  return traverse_expression(expr->cast_expression,line,pos);
    case EXPR_SIZEOF:        return traverse_expression(expr->cast_expression,line,pos);
    case EXPR_PTRSIZEOF:     return traverse_expression(expr->cast_expression,line,pos);
    case EXPR_ALIGNOF:       return ;
    case EXPR_DEREF:         traverse_expression(expr->deref,line,pos); return;
    case EXPR_CALL:          traverse_expression(expr->fn,line,pos);
      { struct expression* ex;
	FOR_EACH_PTR(expr->args, ex) {
	  traverse_expression(ex,line,pos);
	} END_FOR_EACH_PTR(ex);
      } return ;
    case EXPR_SELECT:
    case EXPR_CONDITIONAL:   
      traverse_expression(expr->conditional,line,pos);
      if (!line) {
	traverse_expression(expr->cond_false,line,pos);
	traverse_expression(expr->cond_true,line,pos);
      } else {
	if ((!expr->cond_false) ||
	    (expr->cond_false && !((expr->cond_false->pos.line < line 
				    || ((expr->cond_false->pos.line==line 
					 && expr->cond_false->pos.pos<=pos))))))
	  traverse_expression(expr->cond_false,line,pos);
	else 
	  traverse_expression(expr->cond_true,line,pos);
	return ;
      }
    case EXPR_STATEMENT: traverse_statement(expr->statement,line,pos);  return ;
    case EXPR_LABEL:   return ;
    case EXPR_TYPE:    traverse_expression_symbol(expr,line,pos);  return;
      /* These can not exist as stand-alone expressions */
    case EXPR_INITIALIZER:
      {
	struct expression* ex;
	FOR_EACH_PTR(expr->expr_list, ex) {
	  traverse_expression(ex,line,pos);
	} END_FOR_EACH_PTR(ex);
      }
      return;
    case EXPR_IDENTIFIER:  return   traverse_expression(expr->ident_expression,line,pos);
    case EXPR_INDEX:       return   traverse_expression(expr->idx_expression,line,pos);
    case EXPR_POS:         return   traverse_expression(expr->init_expr,line,pos);
    case EXPR_SLICE:       return   traverse_expression(expr->base,line,pos);
    }
  return ;
}


static void traverse_compound(struct statement* stmt, int line, int pos)
{
  if (!stmt) return;
  if ( stmt->type != STMT_COMPOUND) return;
#ifdef PRINT_DEBUG
  printf("------------------ compound at %s %d:%d\n"
	 ,stream_name(stmt->pos.stream)
	 ,stmt->pos.line
	 ,stmt->pos.pos);
#endif
  struct statement* st;

  struct statement* last_st=0;
  FOR_EACH_PTR(stmt->stmts, st) {
    if (!line) {
      traverse_statement(st,line,pos);
    } else {
      if (st->pos.line < line 
	  || (st->pos.line==line && st->pos.pos<=pos))
	{
	  if (st->type==STMT_COMPOUND)
	    last_st=st;
	  else
	    traverse_statement(st,line,pos);
	}
      else
	{
	  if(last_st)
	    {
	      traverse_statement(last_st,line,pos);
	    }
	}
      //??	return;
    }
  } END_FOR_EACH_PTR(st);
  if (last_st)
    traverse_statement(last_st,line,pos);
    
}


static void traverse_statement(struct statement* stmt, int line, int pos)
{
  if (!stmt) return;
          #ifdef PRINT_DEBUG
                  printf("statement at %s %d:%d\n"
			 ,stream_name(stmt->pos.stream)
			 ,stmt->pos.line
			 ,stmt->pos.pos);
          #endif
  switch ( stmt->type )
    {
    case STMT_DECLARATION:
      {
         #ifdef PRINT_DEBUG
	           printf("traverse declaration at %s %d:%d\n"
			  ,stream_name(stmt->pos.stream)
			  ,stmt->pos.line
			  ,stmt->pos.pos
			  );
         #endif
	struct symbol* s;
	FOR_EACH_PTR(stmt->declaration, s) {
	  add_variable_dec(act_fn,s);
	} END_FOR_EACH_PTR(s);
	break;
      }
    case STMT_COMPOUND:   traverse_compound(stmt,line,pos);	break;
    case STMT_NONE:       break;
    case STMT_EXPRESSION: traverse_expression(stmt->expression,line,pos);break;
    case STMT_IF:
      {
	traverse_expression(stmt->if_conditional,line,pos);
	traverse_statement(stmt->if_true,line,pos);
	traverse_statement(stmt->if_false,line,pos);
	break;
      }
    case STMT_CASE:
      {
	traverse_expression(stmt->case_expression,line,pos);
	traverse_statement(stmt->case_statement,line,pos);
	break;
      }
    case STMT_SWITCH:
      {
	traverse_expression(stmt->switch_expression,line,pos);
	traverse_statement(stmt->switch_statement,line,pos);
	break;
      }
    case STMT_ITERATOR:
      {
	traverse_statement (stmt->iterator_pre_statement,line,pos);
	traverse_expression(stmt->iterator_pre_condition,line,pos);
	traverse_statement (stmt->iterator_statement,line,pos);
	traverse_statement (stmt->iterator_post_statement,line,pos);
	traverse_expression(stmt->iterator_post_condition,line,pos);
	break;
      }
    case STMT_LABEL:
      {
	// struct symbol *label;
	// struct statement *label_statement;
/* 	printf("level=%d ... label at %d:%d\n",level,stmt->pos.line,stmt->pos.pos); */
	break;
      }
    case STMT_GOTO:
      {
/* 	printf("level=%d ... goto at %d:%d\n",level,stmt->pos.line,stmt->pos.pos); */
	break;
      }
    case STMT_ASM:
      {
/* 	printf("level=%d ... asm at %d:%d\n",level,stmt->pos.line,stmt->pos.pos); */
	break;
      }
    case STMT_CONTEXT:
      {
/* 	printf("level=%d ... context at %d:%d\n",level,stmt->pos.line,stmt->pos.pos); */
	break;
      }
    case STMT_RANGE:
      {
	traverse_expression(stmt->range_expression,line,pos);
	traverse_expression(stmt->range_low,line,pos);
	traverse_expression(stmt->range_high,line,pos);
	break;
      }
    case STMT_RETURN:
      {
	break;
      }
    default:
      break;
    }
}

static void traverse_function(struct symbol* sym, int line, int pos)
{
  act_fn=sym;
  if (sym->ctype.base_type 
      && sym->ctype.base_type->arguments
      && sym->ctype.base_type)
    {
      struct symbol* s;
      FOR_EACH_PTR(sym->ctype.base_type->arguments, s) {
	add_variable_dec(act_fn,s);
      } END_FOR_EACH_PTR(s);
      
#if 0
      FOR_EACH_PTR(sym->symbol_list, s) {
	add_variable_dec(act_fn,s);
      } END_FOR_EACH_PTR(s);
#endif

      FOR_EACH_PTR(sym->inline_symbol_list, s) {
	add_variable_dec(act_fn,s);
      } END_FOR_EACH_PTR(s);

    }
  // vypise statement dane funkce
  if (sym->ctype.base_type
      && sym->ctype.base_type->inline_stmt)
    {
      traverse_statement(sym->ctype.base_type->inline_stmt,line,pos);
    }

  if (sym->ctype.base_type
      && sym->ctype.base_type->stmt)
    {      
      traverse_statement(sym->ctype.base_type->stmt,line,pos);
    }
}



static void walk_expression_symbol(struct expression* expr)
{
  // zajisteno jinde
  return;
#ifdef DEBUG_PRINT
  printf("expression-symbol: ");
#endif
  if (expr->symbol)
    {
      struct symbol* sym=expr->symbol->ctype.base_type;
    if (sym)
      {
	if (sym->type==SYM_FN)	  {
	  add_function_use(act_fn,expr); 
	}
	else	  {
	  add_variable_use(act_fn,expr);
	}
      }
    }
  return ;
}
static void walk_expression(struct expression* expr)
{
  if (!expr) return;
  switch (expr->type)
    {
    case EXPR_VALUE:
    case EXPR_FVALUE:   return ;
    case EXPR_STRING:   return ;
    case EXPR_SYMBOL:   walk_expression_symbol(expr);  return;
    case EXPR_BINOP:
    case EXPR_LOGICAL:
    case EXPR_COMMA:
    case EXPR_COMPARE:
    case EXPR_ASSIGNMENT:
	  walk_expression(expr->left);
	  walk_expression(expr->right);
	  return ;
    case EXPR_PREOP:
    case EXPR_POSTOP:     walk_expression(expr->unop);   return ;
    case EXPR_CAST:           return  walk_expression(expr->cast_expression);
    case EXPR_IMPLIED_CAST:   return  walk_expression(expr->cast_expression);
    case EXPR_SIZEOF:         return  walk_expression(expr->cast_expression);
    case EXPR_PTRSIZEOF:      return  walk_expression(expr->cast_expression);
    case EXPR_ALIGNOF:    return ;
    case EXPR_DEREF:      walk_expression(expr->deref);  return ;
    case EXPR_CALL:
      walk_expression(expr->fn);
      {
	struct expression* ex;
	FOR_EACH_PTR(expr->args, ex) {
	  walk_expression(ex);
	} END_FOR_EACH_PTR(ex);
      }
      return ;
    case EXPR_SELECT:
    case EXPR_CONDITIONAL:
      walk_expression(expr->conditional);
      walk_expression(expr->cond_true);
      walk_expression(expr->cond_false);
      return ;
    case EXPR_STATEMENT:   walk_statement(expr->statement,0);   return ;
    case EXPR_LABEL:       return ;
    case EXPR_TYPE:        walk_expression_symbol(expr);        return;
      /* These can not exist as stand-alone expressions */
    case EXPR_INITIALIZER:
      {
	struct expression* ex;
	FOR_EACH_PTR(expr->expr_list, ex) {
	  walk_expression(ex);
	} END_FOR_EACH_PTR(ex);
      }
      return;
    case EXPR_IDENTIFIER:   return   walk_expression(expr->ident_expression);
    case EXPR_INDEX:        return   walk_expression(expr->idx_expression);
    case EXPR_POS:          return   walk_expression(expr->init_expr);
    case EXPR_SLICE:        return   walk_expression(expr->base);
    }
  return ;
}


static void walk_compound(struct statement* stmt, int level)
{
  if (!stmt) return;
  if ( stmt->type != STMT_COMPOUND) return;

  struct statement* st;

  FOR_EACH_PTR(stmt->stmts, st) {
    walk_statement(st,level+1);
  } END_FOR_EACH_PTR(st);
}


static void walk_statement(struct statement* stmt, int level)
{
  if (!stmt) return;
  switch ( stmt->type )
    {
    case STMT_COMPOUND:
      walk_compound(stmt,level);
      break;
    case STMT_NONE:
      {
	break;
      }
    case STMT_EXPRESSION:
      {
	walk_expression(stmt->expression);
	break;
      }
    case STMT_IF:
      {
	walk_expression(stmt->if_conditional);
	walk_statement(stmt->if_true,level+1);
	walk_statement(stmt->if_false,level+1);
	break;
      }
    case STMT_CASE:
      {
	walk_expression(stmt->case_expression);
	walk_statement(stmt->case_statement,level+1);
	break;
      }
    case STMT_SWITCH:
      {
	walk_expression(stmt->switch_expression);
	walk_statement(stmt->switch_statement,level+1);
	break;
      }
    case STMT_ITERATOR:
      {
	walk_statement (stmt->iterator_pre_statement,level+1);
	walk_expression(stmt->iterator_pre_condition);
	walk_statement (stmt->iterator_statement,level+1);
	walk_statement (stmt->iterator_post_statement,level+1);
	walk_expression(stmt->iterator_post_condition);
	break;
      }
    case STMT_LABEL:
      {
	// struct symbol *label;
	// struct statement *label_statement;
/* 	printf("level=%d ... label at %d:%d\n",level,stmt->pos.line,stmt->pos.pos); */
	break;
      }
    case STMT_GOTO:
      {
/* 	printf("level=%d ... goto at %d:%d\n",level,stmt->pos.line,stmt->pos.pos); */
	break;
      }
    case STMT_ASM:
      {
/* 	printf("level=%d ... asm at %d:%d\n",level,stmt->pos.line,stmt->pos.pos); */
	break;
      }
    case STMT_CONTEXT:
      {
/* 	printf("level=%d ... context at %d:%d\n",level,stmt->pos.line,stmt->pos.pos); */
	break;
      }
    case STMT_RANGE:
      {
	walk_expression(stmt->range_expression);
	walk_expression(stmt->range_low);
	walk_expression(stmt->range_high);
	break;
      }
    case STMT_RETURN:
      {
	break;
      }
    default:
      break;
    }
}

static void walk_function(struct symbol* sym)
{
  act_fn=sym;
  if (sym->ctype.base_type && sym->ctype.base_type->arguments)
    {
      struct symbol* s;
      if (sym->ctype.base_type)
	{
	  FOR_EACH_PTR(sym->ctype.base_type->arguments, s) {
	    add_variable_dec(act_fn,s);
	  } END_FOR_EACH_PTR(s);
	}
    }
  // vypise statement dane funkce
  if (sym->ctype.base_type)
    {
      if (sym->ctype.base_type->stmt)
	{
	  walk_statement(sym->ctype.base_type->stmt,0);
	}
      else
	{
	  ; //printf("no statement\n");
	}
    }
}










#endif
